/*************************************************
* Filters Header File                            *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_FILTERS_H__
#define OPENCL_FILTERS_H__

#include "filtbase.h"
#include "symkey.h"

namespace OpenCL {

/*************************************************
* StreamCipherFilter                             *
*************************************************/
template<typename S>
class StreamCipherFilter : public Filter
   {
   public:
      void write(const byte[], u32bit);
      StreamCipherFilter(const StreamCipherKey& key)
         { cipher.set_key(key, key.length()); }
   private:
      static const u32bit BUFFERSIZE = DEFAULT_BUFFERSIZE;
      S cipher;
      SecureBuffer<byte, BUFFERSIZE> buffer;
   };

/*************************************************
* HashFilter                                     *
*************************************************/
template<typename H>
class HashFilter : public Filter
   {
   public:
      void write(const byte input[], u32bit len) { hash.update(input, len); }
      void final();
      HashFilter(u32bit o_len = H::HASHLENGTH) { out_len = o_len; }
   private:
      H hash;
      u32bit out_len;
   };

/*************************************************
* MACFilter                                      *
*************************************************/
template<typename M>
class MACFilter : public Filter
   {
   public:
      void write(const byte input[], u32bit len) { mac.update(input, len); }
      void final();
      MACFilter(const MACKey& key, u32bit outsize = M::MACLENGTH)
         { mac.set_key(key, key.length()); out_len = outsize; }
   private:
      M mac;
      u32bit out_len;
   };

/*************************************************
* EncoderFilter                                  *
*************************************************/
template<typename E>
class EncoderFilter : public Filter
   {
   public:
      void write(const byte[], u32bit);
      void final();
      EncoderFilter(bool breaks = false, u32bit length = 78)
         : LINEBREAKS(breaks), LINELENGTH(length)
         { counter = position = 0; }
   private:
      static const u32bit INPUTSIZE = E::INPUTSIZE, OUTPUTSIZE = E::OUTPUTSIZE;
      void linebreak() { if(LINEBREAKS) send('\n'); counter = 0; }
      void output(const byte[OUTPUTSIZE]);
      void process(byte);
      const bool LINEBREAKS;
      const u32bit LINELENGTH;
      SecureBuffer<byte, INPUTSIZE> inbuffer;
      u32bit counter, position;
   };

/*************************************************
* Decoder Filter                                 *
*************************************************/
template<typename E>
class DecoderFilter : public Filter
   {
   public:
      void write(const byte[], u32bit);
      void final();
      bool is_valid(byte input) { return E::is_valid(input); }
      DecoderFilter() { position = 0; }
   private:
      static const u32bit INPUTSIZE = E::OUTPUTSIZE, OUTPUTSIZE = E::INPUTSIZE;
      void process(byte);
      SecureBuffer<byte, INPUTSIZE> inbuffer;
      u32bit position;
   };

}

#endif

#ifndef OPENCL_FILTERS_ICC__
#define OPENCL_FILTERS_ICC__
#include "filters.icc"
#endif
