/*************************************************
* OpenCL Basic Interface Header File             *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_OPENCL_H__
#define OPENCL_OPENCL_H__

#include "exceptn.h"
#include "secmem.h"

namespace OpenCL {

/*************************************************
* SymmetricKeyedAlgorithm Base Class             *
*************************************************/
struct SymmetricKeyedAlgorithm
   {
   virtual void set_key(const byte[], u32bit) throw(InvalidKeyLength) = 0;
   virtual ~SymmetricKeyedAlgorithm() {}
   };

/*************************************************
* Block Cipher Base Class                        *
*************************************************/
struct BlockCipher : public SymmetricKeyedAlgorithm
   {
   virtual void encrypt(const byte[], byte[]) const = 0;
   virtual void decrypt(const byte[], byte[]) const = 0;
   virtual void encrypt(byte block[]) const = 0;
   virtual void decrypt(byte block[]) const = 0;
   virtual void clear() throw() = 0;
   virtual ~BlockCipher() {}
   };

/*************************************************
* Stream Cipher Base Class                       *
*************************************************/
struct StreamCipher : public SymmetricKeyedAlgorithm
   {
   virtual void encrypt(const byte[], byte[], u32bit) = 0;
   virtual void encrypt(byte[], u32bit) = 0;
   virtual void decrypt(const byte[], byte[], u32bit);
   virtual void decrypt(byte[], u32bit);
   virtual void clear() throw() = 0;
   virtual ~StreamCipher() {}
   };

/*************************************************
* Random Access Stream Cipher Base Class         *
*************************************************/
struct RandomAccessStreamCipher : public StreamCipher
   {
   virtual void seek(u32bit) = 0;
   virtual ~RandomAccessStreamCipher() {}
   };

/*************************************************
* Hash Function Base Class                       *
*************************************************/
struct HashFunction
   {
   virtual void process(const byte[], u32bit, byte[]);
   virtual void update(const byte[], u32bit) = 0;
   virtual void final(byte[]) = 0;
   virtual void clear() throw() = 0;
   virtual ~HashFunction() {}
   };

/*************************************************
* Message Authentication Code Base Class         *
*************************************************/
struct MessageAuthenticationCode : public SymmetricKeyedAlgorithm
   {
   virtual void process(const byte[], u32bit, byte[]);
   virtual void update(const byte[], u32bit) = 0;
   virtual void final(byte[]) = 0;
   virtual void clear() throw() = 0;
   virtual ~MessageAuthenticationCode() {}
   };

/*************************************************
* Random Number Generator Base Class             *
*************************************************/
struct RandomNumberGenerator
   {
   virtual byte random() = 0;
   virtual void randomize(byte[], u32bit);
   virtual void add_entropy(const byte[], u32bit) throw() = 0;
   virtual void add_entropy(class EntropySource&, bool) = 0;
   virtual void clear() throw() = 0;
   virtual ~RandomNumberGenerator() {}
   };

/*************************************************
* Entropy Source Base Class                      *
*************************************************/
struct EntropySource
   {
   virtual u32bit get_entropy(byte[], u32bit) = 0;
   virtual ~EntropySource() {}
   };

}

#endif
